/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JContainer3.java 402 2006-04-24 16:39:29Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.container;

import java.io.File;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Proxy;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.objectweb.easybeans.api.EZBContainer;
import org.objectweb.easybeans.api.EZBContainerCallbackInfo;
import org.objectweb.easybeans.api.EZBContainerException;
import org.objectweb.easybeans.api.EZBContainerLifeCycleCallback;
import org.objectweb.easybeans.api.Factory;
import org.objectweb.easybeans.api.FactoryException;
import org.objectweb.easybeans.container.info.SessionBeanInfo;
import org.objectweb.easybeans.container.session.SessionFactory;
import org.objectweb.easybeans.container.session.stateful.StatefulSessionFactory;
import org.objectweb.easybeans.container.session.stateless.StatelessSessionFactory;
import org.objectweb.easybeans.deployment.Deployment;
import org.objectweb.easybeans.deployment.annotations.exceptions.AnalyzerException;
import org.objectweb.easybeans.deployment.annotations.exceptions.ResolverException;
import org.objectweb.easybeans.deployment.annotations.impl.JLocal;
import org.objectweb.easybeans.deployment.annotations.impl.JRemote;
import org.objectweb.easybeans.deployment.annotations.metadata.ClassAnnotationMetadata;
import org.objectweb.easybeans.deployment.annotations.metadata.EjbJarAnnotationMetadata;
import org.objectweb.easybeans.deployment.helper.JavaContextHelper;
import org.objectweb.easybeans.deployment.helper.JavaContextHelperException;
import org.objectweb.easybeans.deployment.resolver.JNDIResolver;
import org.objectweb.easybeans.deployment.xml.EJB3DeploymentDescException;
import org.objectweb.easybeans.enhancer.Enhancer;
import org.objectweb.easybeans.enhancer.EnhancerException;
import org.objectweb.easybeans.loader.EasyBeansClassLoader;
import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;
import org.objectweb.easybeans.persistence.PersistenceUnitManager;
import org.objectweb.easybeans.persistence.xml.PersistenceXmlFileAnalyzer;
import org.objectweb.easybeans.persistence.xml.PersistenceXmlFileAnalyzerException;
import org.objectweb.easybeans.rpc.ClientRPCInvocationHandler;

/**
 * Defines an EJB3 container.
 * @author Florent Benoit
 */
public class JContainer3 implements EZBContainer {

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(JContainer3.class);

    /**
     * Id of this container.
     */
    private String id = null;

    /**
     * Archive used by this container.
     */
    private File archive = null;

    /**
     * Classloader use to manage this archive.
     */
    private ClassLoader classLoader = null;

    /**
     * Deployment for the managed archive.
     */
    private Deployment deployment = null;

    /**
     * Container available.
     */
    private boolean available = false;

    /**
     * Map of managed ejb3 factories.
     */
    private Map<String, Factory> factories = null;

    /**
     * Persistence manager object which manages all persistence-unit associated to this container.
     */
    private PersistenceUnitManager persistenceUnitManager = null;

    /**
     * Callbacks.
     */
    private List<EZBContainerLifeCycleCallback> callbacksLifeCycle = null;


    /**
     * Build a new container on the given archive.
     * @param archive the archive (jar file or exploded).
     */
    public JContainer3(final File archive) {
        this.archive = archive;
        this.id = String.valueOf(System.identityHashCode(this));
        deployment = new Deployment(archive);
        this.factories = new HashMap<String, Factory>();
        callbacksLifeCycle = new ArrayList<EZBContainerLifeCycleCallback>();
    }

    /**
     * Gets the id of this container.
     * @return string id.
     */
    public String getId() {
        return id;
    }

    /**
     * Start this container.
     * @throws EZBContainerException if starting fails.
     */
    public void start() throws EZBContainerException {

        // Analyze files
        long tStart = System.currentTimeMillis();
        try {
            deployment.analyze();
        } catch (AnalyzerException e) {
            throw new EZBContainerException("Cannot analyze archive '" + archive.getName() + "'.", e);
        } catch (ResolverException e) {
            throw new EZBContainerException(
                    "Cannot resolve some annotations in the archive '" + archive.getName() + "'.", e);
        } catch (EJB3DeploymentDescException e) {
            throw new EZBContainerException(
                    "Cannot parse deployment descriptor in the archive '" + archive.getName() + "'.", e);
        }
        if (logger.isInfoEnabled()) {
            logger.info("Analyze elapsed during : " + (System.currentTimeMillis() - tStart) + " ms");
        }

        // Run enhancer on classes found
        URL url;
        try {
            url = archive.toURL();
        } catch (MalformedURLException e) {
            throw new EZBContainerException("Cannot get URL from archive '" + archive.getName() + "'.", e);
        }

        // Build JNDI resolver
        JNDIResolver jndiResolver = new JNDIResolver();
        jndiResolver.addDeployment(deployment);

        // add resolver in the map
        Map<String, Object> enhancerMap = new HashMap<String, Object>();
        enhancerMap.put(JNDIResolver.NAME, jndiResolver);

        ClassLoader old = Thread.currentThread().getContextClassLoader();
        // Define classloader if it was not yet defined (ear case --> classloader already set)
        if (classLoader == null) {
            classLoader = new EasyBeansClassLoader(new URL[] {url}, old);
        }
        try {
            Thread.currentThread().setContextClassLoader(classLoader);
            Enhancer enhancer = new Enhancer(classLoader, deployment.getAnnotationDeploymentAnalyzer()
                    .getEjbJarAnnotationMetadata(), enhancerMap);

            long tStartEnhancing = System.currentTimeMillis();
            try {
                enhancer.enhance();
            } catch (EnhancerException e) {
                throw new EZBContainerException("Cannot run enhancer on archive '" + archive.getName() + "'.", e);
            }
            if (logger.isInfoEnabled()) {
                logger.info("Enhancement elapsed during : " + (System.currentTimeMillis() - tStartEnhancing) + " ms");
            }

            // Check if there is META-INF/persistence.xml file
            PersistenceUnitManager analyzedPersistenceUnitManager = null;
            try {
                analyzedPersistenceUnitManager = PersistenceXmlFileAnalyzer.analyzePersistenceXmlFile(archive, getClassLoader());
            } catch (PersistenceXmlFileAnalyzerException e) {
                throw new EZBContainerException("Cannot analyze the persistence.xml file in the archive", e);
            }

            // No previous manager
            if (this.persistenceUnitManager == null) {
                this.persistenceUnitManager = analyzedPersistenceUnitManager;
            } else {
                // merge old and new.
                if (analyzedPersistenceUnitManager != null) {
                    analyzedPersistenceUnitManager.merge(persistenceUnitManager);
                    // update persistence manager with the merged one.
                    this.persistenceUnitManager = analyzedPersistenceUnitManager;
                }
            }

            // Send notification to callbacks
            if (callbacksLifeCycle.size() > 0) {
                EZBContainerCallbackInfo info = getContainer3CallbackInfo();
                for (EZBContainerLifeCycleCallback callback : callbacksLifeCycle) {
                    callback.start(info);
                }
            }

            // bind session beans
            EjbJarAnnotationMetadata ejbMetadata = deployment.getAnnotationDeploymentAnalyzer()
                    .getEjbJarAnnotationMetadata();
            if (ejbMetadata != null) {
                for (ClassAnnotationMetadata bean : ejbMetadata.getClassAnnotationMetadataCollection()) {
                    String className = bean.getClassName().replace("/", ".");

                    if (bean.isSession()) {
                        SessionFactory sessionFactory = null;
                        Context javaContext;
                        try {
                            javaContext = JavaContextHelper.build(bean);
                        } catch (JavaContextHelperException e) {
                            throw new EZBContainerException("Cannot build environment", e);
                        }

                        if (bean.isStateless()) {
                            try {
                                sessionFactory = new StatelessSessionFactory(className, this);
                                sessionFactory.setJavaContext(javaContext);
                            } catch (FactoryException fe) {
                                throw new EZBContainerException("Cannot build the stateless factory", fe);
                            }
                        } else {
                            try {
                                sessionFactory = new StatefulSessionFactory(className, this);
                                sessionFactory.setJavaContext(javaContext);
                            } catch (FactoryException fe) {
                                throw new EZBContainerException("Cannot build the stateless factory", fe);
                            }
                        }

                        // build runtime information
                        SessionBeanInfo sessionBeanInfo = new SessionBeanInfo();
                        sessionBeanInfo.setTransactionManagementType(bean.getTransactionManagementType());
                        sessionFactory.setSessionBeanInfo(sessionBeanInfo);

                        factories.put(className, sessionFactory);

                        // get interface of bean
                        JLocal localItfs = bean.getLocalInterfaces();
                        JRemote remoteItfs = bean.getRemoteInterfaces();
                        InvocationHandler handler = new ClientRPCInvocationHandler(getId(), className);

                        if (localItfs != null) {
                            for (String itf : localItfs.getInterfaces()) {
                                bindItf(itf, handler, bean, "Local");
                            }
                        }
                        if (remoteItfs != null) {
                            for (String itf : remoteItfs.getInterfaces()) {
                                bindItf(itf, handler, bean, "Remote");
                            }
                        }
                    }
                }
            }
            // cleanup
            deployment = new Deployment(archive);
            enhancer = null;
        } finally {
            Thread.currentThread().setContextClassLoader(old);
        }
        if (logger.isInfoEnabled()) {
            logger.info("Container started in : " + (System.currentTimeMillis() - tStart) + " ms");
        }
        available = true;

    }


    /**
     * Stop this container.
     */
    public void stop() {
        available = false;

        // stop each factories
        for (Factory f : factories.values()) {
            f.stop();
        }

        // reset CL
        classLoader = null;

        // TODO : no unbind for now
        // need to configure it ?

        // Send notification to callbacks
        if (callbacksLifeCycle.size() > 0) {
            EZBContainerCallbackInfo info = getContainer3CallbackInfo();
            for (EZBContainerLifeCycleCallback callback : callbacksLifeCycle) {
                callback.stop(info);
            }
        }
    }

    /**
     * Gets information on the container that can be given to container
     * callbacks.
     * @return information on the managed container.
     */
    private EZBContainerCallbackInfo getContainer3CallbackInfo() {
        EZBContainerCallbackInfo info = new EZBContainerCallbackInfo();
        info.setArchive(archive);
        return info;
    }

    /**
     * Bind into registry interfaces for a bean making them available from
     * clients.
     * @param itf the name of the interface that object will have.
     * @param handler object which will be called on the client side.
     * @param bean the bean class associated to given interface.
     * @param mode Local, Remote, etc
     * @throws EZBContainerException if interface cannot be loaded or if the bind
     *         fails
     */
    private void bindItf(final String itf, final InvocationHandler handler,
            final ClassAnnotationMetadata bean, final String mode) throws EZBContainerException {
        String itfClsName = itf.replace("/", ".");
        Class clz = null;
        try {
            clz = classLoader.loadClass(itfClsName);
        } catch (ClassNotFoundException e) {
            throw new EZBContainerException("Cannot find the class '" + itf + "' in Classloader '" + classLoader + "'.");
        }
        Object o = Proxy.newProxyInstance(classLoader, new Class[] {clz}, handler);
        try {
            String jndiName = jndiNameEncode(bean.getClassName(), itfClsName, mode);
            logger.debug("Binding bean {0} with interface {1} into registry with jndi name {2}",
                    bean.getClassName().replace("/", "."), itf.replace("/", "."), jndiName);
            new InitialContext().rebind(jndiName, o);
        } catch (NamingException e) {
            throw new EZBContainerException("Cannot bind the object '" + o + "' into JNDI with the name '"
                    + clz.getName() + "'.", e);
        }

    }

    /**
     * Gets a factory with its given name.
     * @param factoryName the factory name.
     * @return the factory found or null.
     */
    public Factory getFactory(final String factoryName) {
        return factories.get(factoryName);
    }

    /**
     * Gets the name of this container.
     * @return the name.
     */
    public String getName() {
        return archive.getName();
    }

    /**
     * Gets the classloader.
     * @return classloader of the container
     */
    public ClassLoader getClassLoader() {
        return classLoader;
    }

    /**
     * Gets the archive used by this container. It can be a .jar file or a
     * directory.
     * @return the archive.
     */
    public File getArchive() {
        return archive;
    }

    /**
     * Check if the container is available or not.
     * @return true if the container is available.
     */
    public boolean isAvailable() {
        return available;
    }

    /**
     * Gets the persistence manager object which manages all persistence-unit associated to this container.
     * @return persistence unit manager object
     */
    public PersistenceUnitManager getPersistenceUnitManager() {
        return persistenceUnitManager;
    }


    /**
     * Encode a given name by prefixing it with given mode.
     * @param beanClassName name of the bean class.
     * @param itfName name of the interface.
     * @param mode local/remote, etc
     * @return encoded jndi Name
     */
    public static String jndiNameEncode(final String beanClassName, final String itfName, final String mode) {
        return beanClassName.replace("/", ".") + "_" + itfName.replace("/", ".") + "@" + mode;
    }

    /**
     * Sets the classloader.
     * @param classLoader to be used by the container
     */
    public void setClassLoader(final ClassLoader classLoader) {
        if (this.classLoader != null) {
            throw new IllegalArgumentException("Cannot replace an existing classloader");
        }
        this.classLoader = classLoader;
    }

    /**
     * Sets the persistence manager object which manages all persistence-unit associated to this container.
     * @param persistenceUnitManager persistence unit manager object to set.
     */
    public void setPersistenceUnitManager(final PersistenceUnitManager persistenceUnitManager) {
        if (this.persistenceUnitManager != null) {
            throw new IllegalArgumentException("Cannot replace an existing persistenceUnitManager");
        }
        this.persistenceUnitManager = persistenceUnitManager;
    }


}
