/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: SessionFactory.java 173 2006-03-10 14:49:41Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.container.session;

import java.lang.reflect.Method;
import java.util.Map;

import javax.naming.Context;
import javax.naming.NamingException;

import org.objectweb.easybeans.api.EZBContainer;
import org.objectweb.easybeans.api.FactoryException;
import org.objectweb.easybeans.api.bean.IBeanInfo;
import org.objectweb.easybeans.container.AbsFactory;
import org.objectweb.easybeans.container.info.SessionBeanInfo;
import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;
import org.objectweb.easybeans.naming.NamingManager;
import org.objectweb.easybeans.pool.Pool;
import org.objectweb.easybeans.pool.PoolException;
import org.objectweb.easybeans.rpc.util.Hash;

/**
 * This class manages the session bean and its creation/lifecycle.
 * @param <PoolType> the type of bean instance.
 * @author Florent Benoit
 */
public abstract class SessionFactory<PoolType> extends AbsFactory {

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(SessionFactory.class);

    /**
     * Name of the class of the managed bean.
     */
    private String className = null;

    /**
     * Container that created this factory.
     */
    private EZBContainer container = null;

    /**
     * Keep a direct reference to the method so that we don't need to compute
     * each time the method object to invoke.<br>
     * http://java.sun.com/j2se/1.5.0/docs/guide/rmi/spec/rmi-stubs24.html
     */
    private Map<Long, Method> hashes = null;

    /**
     * Pool that manage beans instance.
     */
    private Pool<PoolType, Long> pool = null;

    /**
     * Class used to build bean's instance.
     */
    private Class<PoolType> beanClass = null;

    /**
     * Context for java: lookups.
     */
    private Context javaContext = null;

    /**
     * Reference on the naming manager.
     */
    private static NamingManager namingManager = null;

    /**
     * Session Desc (deployment info).
     */
    private SessionBeanInfo sessionBeanInfo = null;

    /**
     * Gets the java: context.
     * @return java: context.
     */
    public Context getJavaContext() {
        return javaContext;
    }

    /**
     * Sets the java: context.
     * @param javaContext the java: context.
     */
    public void setJavaContext(final Context javaContext) {
        this.javaContext = javaContext;
    }

    /**
     * Gets the bean's class.
     * @return bean class used to instantiate beans.
     */
    protected Class<PoolType> getBeanClass() {
        return beanClass;
    }

    /**
     * Sets the bean class that will be used to build bean's instance.
     * @param beanClass the instance of the bean class name
     */
    protected void setBeanClass(final Class<PoolType> beanClass) {
        this.beanClass = beanClass;
    }

    /**
     * Sets the pool used by this factory.
     * @param pool the pool which managed bean instances
     */
    protected void setPool(final Pool<PoolType, Long> pool) {
        this.pool = pool;
    }

    /**
     * Gets the computed hashes.
     * @return computed hashes
     */
    protected Map<Long, Method> getHashes() {
        return hashes;
    }

    /**
     * Sets the hashes for the current bean class.
     * @param hashes method hashes computed as RMI hashes
     */
    protected void setHashes(final Map<Long, Method> hashes) {
        this.hashes = hashes;
    }

    /**
     * Builds a new factory with a given name and its container.
     * @param className name of this factory (name of class that is managed)
     * @param container the root component of this factory.
     * @throws FactoryException if class can't be loaded.
     */
    @SuppressWarnings("unchecked")
    public SessionFactory(final String className, final EZBContainer container) throws FactoryException {
        this.className = className;
        this.container = container;
        Class clazz = null;
        try {
            clazz = getContainer().getClassLoader().loadClass(getClassName());
        } catch (ClassNotFoundException e) {
            throw new FactoryException("Cannot load the class for class name '" + getClassName() + "'", e);
        }
        setBeanClass(clazz);
        setHashes(Hash.hashClass(clazz));

        try {
            namingManager = NamingManager.getInstance();
        } catch (NamingException e) {
            throw new FactoryException("Cannot get instance of the naming manager", e);
        }
    }

    /**
     * Gets the reference on the naming manager.
     * @return the reference on the naming manager.
     */
    protected static NamingManager getNamingManager() {
        return namingManager;
    }

    /**
     * Stops the factory.
     */
    public void stop() {
        // remove pool
        try {
            pool.stop();
        } catch (PoolException e) {
            logger.error("Problem when stopping the factory", e);
        }

    }

    /**
     * Gets the container used by this factory.
     * @return container of this factory
     */
    public EZBContainer getContainer() {
        return container;
    }

    /**
     * Gets the className used by this factory.
     * @return classname that will be instantiated to build bean instance.
     */
    public String getClassName() {
        return className;
    }

    /**
     * Gets the pool used by this factory.
     * @return pool.
     */
    protected Pool<PoolType, Long> getPool() {
        return pool;
    }

    /**
     * @return information of the current bean.
     */
    public IBeanInfo getBeanInfo() {
        return sessionBeanInfo;
    }

    /**
     * @return information of the current bean.
     */
    public SessionBeanInfo getSessionBeanInfo() {
        return sessionBeanInfo;
    }

    /**
     * Sets the information object for a session bean.
     * @param sessionBeanInfo information on the bean.
     */
    public void setSessionBeanInfo(final SessionBeanInfo sessionBeanInfo) {
        this.sessionBeanInfo = sessionBeanInfo;
    }
}
