/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EJB3DeploymentDescLoader.java 133 2006-03-06 18:00:57Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.deployment.xml.parsing;

import java.net.URL;

import org.objectweb.easybeans.deployment.xml.struct.AbsBean;
import org.objectweb.easybeans.deployment.xml.struct.EJB3;
import org.objectweb.easybeans.deployment.xml.struct.EnterpriseBeans;
import org.objectweb.easybeans.deployment.xml.struct.Session;
import org.objectweb.easybeans.deployment.xml.struct.common.AbsEnvironment;
import org.objectweb.easybeans.deployment.xml.struct.common.EnvEntry;
import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;
import org.objectweb.easybeans.util.xml.DocumentParser;
import org.objectweb.easybeans.util.xml.DocumentParserException;
import org.objectweb.easybeans.util.xml.XMLUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * This class analyzes the EJB3 deployment descriptor and fill a struct with
 * getters.
 * @author Florent Benoit
 */
public final class EJB3DeploymentDescLoader {

    /**
     * Persistence namespace.
     */
    private static final String JAVAEE_NS = "http://java.sun.com/xml/ns/javaee";

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(EJB3DeploymentDescLoader.class);

    /**
     * Validating with schema ?
     */
    private static boolean validating = true;

    /**
     * Utility class, no constructor.
     */
    private EJB3DeploymentDescLoader() {

    }

    /**
     * Load the ejb-jar.xml file.
     * @param url the URL of the XML file.
     * @throws ParsingException if parsing of XML file fails.
     * @return an application object.
     */
    public static EJB3 loadDeploymentDescriptor(final URL url) throws ParsingException {
        logger.debug("Analyzing url {0}", url);

        // Get document
        Document document = null;
        try {
            document = DocumentParser.getDocument(url, validating, new EJB3EntityResolver());
        } catch (DocumentParserException e) {
            throw new ParsingException("Cannot parse the url", e);
        }

        EJB3 ejb3 = new EJB3();

        // Root element = <ejb-jar>
        Element ejbJarRootElement = document.getDocumentElement();

        NodeList enterprisebeansList = ejbJarRootElement.getElementsByTagNameNS(JAVAEE_NS, EnterpriseBeans.NAME);
        if (enterprisebeansList.getLength() == 1) {
            Element enterpriseBeansElement = (Element) enterprisebeansList.item(0);

            // Create struct
            EnterpriseBeans enterpriseBeans = new EnterpriseBeans();
            ejb3.setEnterpriseBeans(enterpriseBeans);

            // Analyze beans
            analyzeBeans(enterpriseBeansElement, enterpriseBeans);

        }

        return ejb3;

    }

    /**
     * Analyze common stuff of beans.
     * @param enterpriseBeansElement the dom element.
     * @param enterpriseBeans the structure representing object.
     */
    private static void analyzeBeans(final Element enterpriseBeansElement, final EnterpriseBeans enterpriseBeans) {
        // Get session beans
        NodeList sessionBeansList = enterpriseBeansElement.getElementsByTagNameNS(JAVAEE_NS, Session.NAME);

        // Loop on session beans
        for (int i = 0; i < sessionBeansList.getLength(); i++) {
            Element sessionElement = (Element) sessionBeansList.item(i);

            // Build instance that is created.
            Session session = new Session();

            enterpriseBeans.addSession(session);

            // First, analyze common things
            analyzeBean(sessionElement, session);
        }

    }

    /**
     * Analyze common stuff of a bean.
     * @param beanElement the dom element.
     * @param bean the structure representing object.
     */
    private static void analyzeBean(final Element beanElement, final AbsBean bean) {
        // Get ejb-name
        String ejbName = XMLUtils.getStringValueElement(JAVAEE_NS, beanElement, "ejb-name");
        bean.setEjbName(ejbName);

        // Get ejb-class
        String ejbClass = XMLUtils.getStringValueElement(JAVAEE_NS, beanElement, "ejb-class");
        bean.setEjbClass(ejbClass);

        // Analyze common things

        analyzeEnvironement(beanElement, bean);
    }

    /**
     * Analyze the environment (could be a bean, servlet, etc).
     * @param envElement the dom element.
     * @param env the structure representing object.
     */
    private static void analyzeEnvironement(final Element envElement, final AbsEnvironment env) {
        analyzeEnvEnvtry(envElement, env);
    }

    /**
     * Analyze the env-entry element.
     * @param envElement the dom element.
     * @param env the structure representing object.
     */
    private static void analyzeEnvEnvtry(final Element envElement, final AbsEnvironment env) {
        // Get env-entry
        NodeList envEntryList = envElement.getElementsByTagNameNS(JAVAEE_NS, EnvEntry.NAME);

        // Loop on env entry
        for (int i = 0; i < envEntryList.getLength(); i++) {
            Element envEntryElement = (Element) envEntryList.item(i);

            // Build instance that is created.
            EnvEntry envEntry = new EnvEntry();
            env.addEnvEntry(envEntry);

            // Env-entry name
            String envEntryName = XMLUtils.getStringValueElement(JAVAEE_NS, envEntryElement, "env-entry-name");
            envEntry.setEnvEntryName(envEntryName);

            // Env-entry type
            String envEntryType = XMLUtils.getStringValueElement(JAVAEE_NS, envEntryElement, "env-entry-type");
            envEntry.setEnvEntryType(envEntryType);

            // Env-entry value
            String envEntryValue = XMLUtils.getStringValueElement(JAVAEE_NS, envEntryElement, "env-entry-value");
            envEntry.setEnvEntryValue(envEntryValue);
        }
    }

}
