/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ContextHelper.java 363 2006-04-19 14:40:11Z studzine $
 * --------------------------------------------------------------------------
 */
package org.objectweb.easybeans.tests.common.helper;

import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 * Used to do common operations on bean contexts.
 * @author Eduardo Studzinski Estima de Castro
 * @author Gisele Pinheiro Souza
 */
public final class ContextHelper {

    /**
     * Error message.
     */
    public static final String ERROR_MSG_INJECTION = "The container did not inject the default value specified "
        + "in the descriptor.";

    /**
     * Error message.
     */
    public static final String ERROR_MSG_SESSION_CONTEXT = "Error in access using the Session Context.";

    /**
     * Error message.
     */
    public static final String ERROR_MSG_JNDI_ACCESS = "Error in access using the JNDI API directly.";

    /**
     * Creates a new instance of ContextHelper.
     */
    private ContextHelper() {
    }

    /**
     * Checks the following items:<li>if an environment entry value specified
     * in the descriptor was correctly injected;</li> <li>if the session
     * context can be used to access the environment entry;</li> <li>if the
     * JNDI API can be used directly to access the environment entry;</li> If
     * fails, an IllegalStateException is thrown.
     * @param <E> Element Type
     * @param sessionContext Session Context instance
     * @param entryName Entry name in the environment.
     * @param beanValue Value obtained in the bean object.
     * @param expectedValue Value expected by the element.
     */
    @SuppressWarnings("unchecked")
    public static <E> void checkInjection(final SessionContext sessionContext, final String entryName,
            final E beanValue, final E expectedValue) {

        // Injection
        if (!expectedValue.equals(beanValue)) {
            throw new IllegalStateException(ERROR_MSG_INJECTION);
        }

        checkEnvironment(sessionContext, entryName, expectedValue);
    }

    /**
     * Checks the following items:<li>if the session context can be used to
     * access the environment entry;</li> <li>if the JNDI API can be used
     * directly to access the environment entry;</li> If fails, an
     * IllegalStateException is thrown.
     * @param <E> Element Type
     * @param sessionContext Session Context instance
     * @param entryName Entry name in the environment.
     * @param beanInterface Bean interface.
     */
    @SuppressWarnings("unchecked")
    public static <E> void checkBeanRefEnvironment(final SessionContext sessionContext, final String entryName,
            final E beanInterface) {

        // Session Context
        E sctxEntry = (E) sessionContext.lookup(entryName);

        if (sctxEntry == null) {
            throw new IllegalStateException(ERROR_MSG_SESSION_CONTEXT);
        }
        // JNDI Access
        try {
            Context initCtx = new InitialContext();
            Context myEnv = (Context) initCtx.lookup("java:comp/env");

            E eJNDI = (E) myEnv.lookup(entryName);

            if (eJNDI == null) {
                throw new IllegalStateException(ERROR_MSG_JNDI_ACCESS);
            }
        } catch (NamingException e) {
            throw new IllegalStateException("The context could not be obtained.");
        }
    }

    /**
     * Checks the following items:<li>if the session context can be used to
     * access the environment entry;</li> <li>if the JNDI API can be used
     * directly to access the environment entry;</li> If fails, an
     * IllegalStateException is thrown.
     * @param <E> Element Type
     * @param sessionContext Session Context instance
     * @param entryName Entry name in the environment.
     * @param expectedValue Value expected by the element.
     */
    @SuppressWarnings("unchecked")
    public static <E> void checkEnvironment(final SessionContext sessionContext, final String entryName,
            final E expectedValue) {

        // Session Context
        E sctxValue = (E) sessionContext.lookup(entryName);
        if (!sctxValue.equals(expectedValue)) {
            throw new IllegalStateException(ERROR_MSG_SESSION_CONTEXT);
        }

        // JNDI Access
        try {
            Context initCtx = new InitialContext();
            Context myEnv = (Context) initCtx.lookup("java:comp/env");

            E eJNDI = (E) myEnv.lookup(entryName);

            if (!eJNDI.equals(expectedValue)) {
                throw new IllegalStateException(ERROR_MSG_JNDI_ACCESS);
            }
        } catch (NamingException e) {
            throw new IllegalStateException("The context could not be obtained.");
        }
    }
}
