/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TestContainerTransactionException.java 364 2006-04-20 11:41:05Z pinheirg $
 * --------------------------------------------------------------------------
 */
package org.objectweb.easybeans.tests.transaction.containermanaged.base;

import static org.testng.Assert.fail;

import java.sql.SQLException;

import javax.ejb.EJBException;
import javax.naming.NamingException;
import javax.transaction.Status;
import javax.transaction.SystemException;
import javax.transaction.UserTransaction;

import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;
import org.objectweb.easybeans.tests.common.ejbs.base.ItfContainerTransaction;
import org.objectweb.easybeans.tests.common.ejbs.stateless.containermanaged.ItfDatabaseManager;
import org.objectweb.easybeans.tests.common.ejbs.stateless.containermanaged.ItfTransactionMisc00;
import org.objectweb.easybeans.tests.common.ejbs.stateless.containermanaged.SLSBDatabaseManager;
import org.objectweb.easybeans.tests.common.exception.AppException;
import org.objectweb.easybeans.tests.common.exception.RollbackApplicationException;
import org.objectweb.easybeans.tests.common.helper.EJBHelper;
import org.objectweb.easybeans.tests.common.helper.EmbeddedHelper;
import org.objectweb.easybeans.tests.common.helper.TransactionHelper;

/**
 * Verifies if the container manages well the different types of exception that can occur
 * in the transaction context.
 * @author Gisele Pinheiro Souza
 * @author Eduardo Studzinski Estima de Castro
 */
public class TestContainerTransactionException {

    /**
     * Indicates that the method throws an application exception.
     */
    public static final int APP_EXCEPTION = 1;

    /**
     * Indicates that the method throws an application exception with elemente rollback = true.
     */
    public static final int APP_ROLLBACK_EXCEPTION = 2;

    /**
     * Indicates that the method throws a runtime exception.
     */
    public static final int RUNTIME_EXCEPTION = 3;

    /**
     * Constant that defines the first database name.
     */
    protected static final String DATABASE_1 = "jdbc_1";

    /**
     * Constant that defines the second database name.
     */
    protected static final String DATABASE_2 = "jdbc_2";

    /**
     * Bean used during the tests that throws an application exception with
     * rollback = false.
     */
    private ItfContainerTransaction sfsbContainerTransactionApp = null;

    /**
     * Bean used during the tests that throws an application exception with
     * rollback = true.
     */
    private ItfContainerTransaction sfsbContainerTransactionRollback = null;

    /**
     * Bean used during the tests that throws a runtime exception.
     */
    private ItfContainerTransaction sfsbContainerTransactionRuntime = null;

    /**
     * Bean used to manage the database in the server side.
     */
    private ItfDatabaseManager slsbDatabaseManager;


    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(TestContainerTransactionException.class);

    /**
     * Verifies if the table was created in the database.
     * @param database the database name.
     * @param tableName the table name verified.
     * @throws NamingException if a lookup error occurs.
     * @throws SQLException if a database error occurs.
     */
    protected void verifyTable(final String database, final String tableName) throws NamingException, SQLException {
        slsbDatabaseManager.verifyTable(database, tableName);
    }

    /**
     * Initiates all thing needed to execute the tests.
     * @throws Exception if an error occurs.
     */
    public void setup() throws Exception {
        // Inserts all database before execute the test
        // used because the container does not provide this feature yet
        EmbeddedHelper.bindDatasource();
        // creates the bean used to manages the databse in the server site.
        slsbDatabaseManager = EJBHelper.getBeanRemoteInstance(SLSBDatabaseManager.class, ItfDatabaseManager.class);
        //cleans the transaction active in other test
        cleanTransaction();
    }

    /**
     * Gets a new instance of the bean used in the application exception (with
     * rollback=false) tests.
     * @param beanClass the bean class name used to make the lookup.
     * @throws Exception if a lookup error occurs.
     */
    public void createBeanApp(final Class beanClass) throws Exception {
        sfsbContainerTransactionApp = EJBHelper.getBeanRemoteInstance(beanClass, ItfContainerTransaction.class);
    }

    /**
     * Gets a new instance of the bean used in the application exception (with
     * rollback=true) tests.
     * @param beanClass the bean class name used to make the lookup.
     * @throws Exception if a lookup error occurs.
     */
    public void createBeanRollback(final Class beanClass) throws Exception {
        sfsbContainerTransactionRollback = EJBHelper.getBeanRemoteInstance(beanClass, ItfContainerTransaction.class);
    }

    /**
     * Gets a new instance of the bean used in the runtime exception tests.
     * @param beanClass the bean class name used to make the lookup.
     * @throws Exception if a lookup error occurs.
     */
    public void createBeanRuntime(final Class beanClass) throws Exception {
        sfsbContainerTransactionRuntime = EJBHelper.getBeanRemoteInstance(beanClass, ItfContainerTransaction.class);
    }

    /**
     * Calls the bean method toString() to verify if the bean was discarded.
     * @param bean the bean that is verified.
     * @return true if the bean is discarded and false otherwise.
     */
    protected boolean isDiscarded(final ItfContainerTransaction bean) {
        boolean bolResult = false;
        try {
            bean.toString();
        } catch (Exception e) {
            bolResult = true;
            logger.debug("The bean threw an expected error during the execution {0}", e);
        }
        return bolResult;
    }

    /**
     * Verifies if the container re-throws the application exception(with
     * rollback = true) and marks the transaction for rollback in this case.
     * @throws Exception if an error during the tests occurs.
     */
    public void testNotUsingClientTransWithAppRollbackException() throws Exception {
        try {
            sfsbContainerTransactionRollback.insertCorrectFirstErrorSecond(DATABASE_1, DATABASE_2);
            fail("The container did not re-throw the application exception.");
        } catch (RollbackApplicationException e) {
            logger.debug("The bean threw an expected error during the execution {0}", e);
        }
        // verifies if the table was created in the DATABASE_1
        verifyTable(DATABASE_1, ItfContainerTransaction.TABLE);
    }

    /**
     * Verifies if the container re-throws the application exception(with rollback = false) and does
     * not do the rollback in this case.
     * @throws Exception if an error during the tests occurs.
     */
    public void testNotUsingClientTransWithAppException() throws Exception {
        try {
            sfsbContainerTransactionApp.insertCorrectFirstErrorSecond(DATABASE_1, DATABASE_2);
            fail("The container did not re-throw the application exception.");
        } catch (AppException e) {
            logger.debug("The bean threw an expected error during the execution {0}", e);
        }
        // verifies if the table was created in the DATABASE_1
        verifyTable(DATABASE_1, ItfContainerTransaction.TABLE);
    }


    /**
     * Verifies if the container throws the EJBException when the bean throws a RUntimeException and does
     * the rollback in this case.
     * @throws Exception if an error during the tests occurs.
     */
    public void testNotUsingClientTransWithRuntimeException() throws Exception {
        //verifies if the container threw the correct exception
        try {
            sfsbContainerTransactionRuntime.insertCorrectFirstErrorSecond(DATABASE_1, DATABASE_2);
            fail("The container did not throw the EJBException.");
        } catch (EJBException e) {
            logger.debug("The bean threw an expected error during the execution {0}", e);
        }

        // verifies if the container discarded the instance.
        if (!isDiscarded(sfsbContainerTransactionRuntime)) {
            fail("The bean was not discarded.");
        }

        // TODO - verifies if the container log the error.

        // verifies if the table was created in the DATABASE_1
        verifyTable(DATABASE_1, ItfContainerTransaction.TABLE);
    }

    /**
     * Verifies if the container manages the transaction among divers beans.The
     * first bean creates a table in the first database and calls other
     * bean(with transaction attribute REQUIRED) to insert a table in the second
     * database. After the both insertions, the first bean throws a RuntimeException
     * exception. Also, verifies if the container throws an EJBException, discards the bean and does the rollback in
     * this case(if the transaction atttribute of the first bean has a
     * transaction context).
     * @throws Exception if an error during the tests occurs.
     */
    public void testCallOtherBeanReq() throws Exception {
        try {
            sfsbContainerTransactionRuntime.insertTablesUsingAuxBeanReq(DATABASE_1, DATABASE_2);
            fail("The container did not throw the EJBException.");
        } catch (EJBException e) {
            logger.debug("The bean threw an expected error during the execution {0}", e);
        }
        // verifies if the container discarded the instance.
        if (!isDiscarded(sfsbContainerTransactionRuntime)) {
            fail("The bean was not discarded.");
        }

        // TODO - verifies if the container log the error.
    }

    /**
     * Verifies if the container manages the transaction among divers beans.The
     * first bean creates a table in the first database and calls other
     * bean(with transaction attribute NOT_SUPPORTS) to insert a table in the second
     * database. After the both insertions, the first bean throws a RuntimeException
     * exception. Also, verifies if the container throws an EJBException, discards the bean and does the rollback in
     * this case(if the transaction atttribute of the first bean has a
     * transaction context).
     * @throws Exception if an error during the tests occurs.
     */
    public void testCallOtherBeanNotSup() throws Exception {
        try {
            sfsbContainerTransactionRuntime.insertTablesUsingAuxBeanNotSup(DATABASE_1, DATABASE_2);
            fail("The container did not throw the EJBException.");
        } catch (EJBException e) {
            logger.debug("The bean threw an expected error during the execution {0}", e);
        }
        // verifies if the container discarded the instance.
        if (!isDiscarded(sfsbContainerTransactionRuntime)) {
            fail("The bean was not discarded.");
        }

        // TODO - verifies if the container log the error.
    }

    /**
     * Deletes the tables created during test.
     *
     */
    public void deleteTable() {
        // deletes the table after each test to avoid errors.
        deleteTable(DATABASE_1, ItfContainerTransaction.TABLE);
        deleteTable(DATABASE_2, ItfContainerTransaction.TABLE);
        // deletes the auxiliar bean table
        deleteTable(DATABASE_1, ItfTransactionMisc00.TABLE);
        deleteTable(DATABASE_2, ItfTransactionMisc00.TABLE);
    }

    /**
     * Deletes the table in the database.
     * @param dbName the database name in the registry.
     * @param tableName the table name.
     */
    private void deleteTable(final String dbName, final String tableName) {
        // deletes the table after each test to avoid errors.
        try {
            slsbDatabaseManager.deleteTable(dbName, tableName);
        } catch (SQLException e) {
            logger.debug("The table delete threw an error during the execution {0}", e);
        } catch (NamingException e) {
            logger.debug("The table delete threw an error during the execution {0}", e);
        }
    }

    /**
     * Returns the bean that throws a runtime exception.
     * @return the bean.
     */
    public ItfContainerTransaction getRuntimeBean(){
        return sfsbContainerTransactionRuntime;
    }

    /**
     * Makes a rollback in the transaction that is active.
     * @throws Exception if an error occurs during the rollback.
     */
    public void cleanTransaction() throws Exception {
        UserTransaction utx = TransactionHelper.getUserTransaction();
        try {
            if (transactionIsActive()) {
                utx.rollback();
            }
        } catch (Exception e) {
            throw new Exception("Cannot clean the transaction. The test cannot be started");
        }
    }

    /**
     * Verifies if the transaction in the client side is active.
     * @return true if the transaction is active, false otherwise.
     * @throws SystemException if a transaction exception occurs.
     * @throws NamingException if a lookup error occurs.
     */
    protected boolean transactionIsActive() throws SystemException, NamingException {
        UserTransaction utx = TransactionHelper.getUserTransaction();
        boolean bolResult = false;
        if (utx != null) {
            if (utx.getStatus() == Status.STATUS_ACTIVE) {
                bolResult = true;
            }
        }
        return bolResult;
    }

    /**
     * Gets a new transaction.
     * @throws NamingException if a lookup error occurs.
     * @return the class UserTransaction.
     */
    protected UserTransaction getUserTransaction() throws NamingException {
        UserTransaction utx = TransactionHelper.getUserTransaction();
        return utx;
    }
}
